using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;

using CMS.SettingsProvider;
using CMS.DirectoryUtilities;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.SiteProvider;
using CMS.PortalEngine;
using CMS.VirtualPathHelper;

/// <summary>
/// Summary description for SiteManagerFunctions
/// </summary>
public static class SiteManagerFunctions
{

    #region "Transformation management"

    /// <summary>
    /// Checks out the specified transformation to the disk
    /// </summary>
    /// <param name="transformationId">Transformation ID</param>
    public static void CheckOutTransformation(int transformationId)
    {
        string fullName = TransformationInfoProvider.GetTransformationFullName(transformationId);
        TransformationInfo ti = TransformationInfoProvider.GetTransformation(fullName);
        if (ti != null)
        {
            string filename = TransformationInfoProvider.GetTransformationUrl(ti.TransformationFullName, null);

            SaveCodeFile(filename, CMS.VirtualPathHelper.VirtualPathHelper.TransformationDirectives + ti.TransformationCode);

            // Set the stylesheet data
            ti.TransformationCheckedOutByUserID = CMSContext.CurrentUser.UserID;
            ti.TransformationCheckedOutMachineName = HttpContext.Current.Server.MachineName;
            ti.TransformationCheckedOutFilename = filename;
            ti.TransformationVersionGUID = Guid.NewGuid().ToString();
            
            TransformationInfoProvider.SetTransformation(ti);
        }
    }


    /// <summary>
    /// Checks in the specified transformation from the disk
    /// </summary>
    /// <param name="transformationId">Transformation ID</param>
    public static void CheckInTransformation(int transformationId)
    {
        string fullName = TransformationInfoProvider.GetTransformationFullName(transformationId);
        TransformationInfo ti = TransformationInfoProvider.GetTransformation(fullName);
        if (ti != null)
        {
            string filename = HttpContext.Current.Server.MapPath(ti.TransformationCheckedOutFilename);
            TextReader tr = new StreamReader(filename);

            // Read away the directive lines
            int skiplines = CMS.VirtualPathHelper.VirtualPathHelper.TransformationDirectives.Split('\n').Length - 1;
            for (int i = 0; i < skiplines; i++)
            {
                tr.ReadLine();
            }

            string newcode = tr.ReadToEnd();
            tr.Close();
            File.Delete(filename);

            ti.TransformationCheckedOutByUserID = 0;
            ti.TransformationCheckedOutFilename = "";
            ti.TransformationCheckedOutMachineName = "";
            ti.TransformationCode = newcode;
            ti.TransformationVersionGUID = Guid.NewGuid().ToString();
            
            TransformationInfoProvider.SetTransformation(ti);
        }
    }


    /// <summary>
    /// Undo the transformation CheckOut
    /// </summary>
    /// <param name="transformationId">Transformation ID</param>
    public static void UndoCheckOutTransformation(int transformationId)
    {
        string fullName = TransformationInfoProvider.GetTransformationFullName(transformationId);
        TransformationInfo ti = TransformationInfoProvider.GetTransformation(fullName);
        if (ti != null)
        {
            string filename = HttpContext.Current.Server.MapPath(ti.TransformationFullName);
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }

            ti.TransformationCheckedOutByUserID = 0;
            ti.TransformationCheckedOutFilename = "";
            ti.TransformationCheckedOutMachineName = "";
            ti.TransformationVersionGUID = Guid.NewGuid().ToString();
            
            TransformationInfoProvider.SetTransformation(ti);
        }
    }

    #endregion


    #region "CSS Stylesheets management"

    /// <summary>
    /// Checks out the specified CSS stylesheet to the disk
    /// </summary>
    /// <param name="cssStylesheetId">Stylesheet ID</param>
    public static void CheckOutStylesheet(int cssStylesheetId)
    {
        CssStylesheetInfo si = CssStylesheetInfoProvider.GetCssStylesheetInfo(cssStylesheetId);
        if (si != null)
        {
            string filename = CssStylesheetInfoProvider.GetVirtualStylesheetUrl(si.StylesheetName, null);

            SaveCodeFile(filename, si.StylesheetText);

            // Set the stylesheet data
            si.StylesheetCheckedOutByUserID = CMSContext.CurrentUser.UserID;
            si.StylesheetCheckedOutMachineName = HttpContext.Current.Server.MachineName;
            si.StylesheetCheckedOutFilename = filename;
            si.StylesheetVersionGUID = Guid.NewGuid().ToString();

            CssStylesheetInfoProvider.SetCssStylesheetInfo(si);
        }
    }


    /// <summary>
    /// Checks-in the CSS stylesheet from the file
    /// </summary>
    /// <param name="cssStylesheetId">Stylesheet ID</param>
    public static void CheckInStylesheet(int cssStylesheetId)
    {
        CssStylesheetInfo si = CssStylesheetInfoProvider.GetCssStylesheetInfo(cssStylesheetId);
        if (si != null)
        {
            string filename = HttpContext.Current.Server.MapPath(si.StylesheetCheckedOutFilename);
            TextReader tr = new StreamReader(filename);

            string newcode = tr.ReadToEnd();
            tr.Close();
            tr.Dispose();
            File.Delete(filename);

            si.StylesheetCheckedOutByUserID = 0;
            si.StylesheetCheckedOutFilename = "";
            si.StylesheetCheckedOutMachineName = "";
            si.StylesheetText = newcode;
            si.StylesheetVersionGUID = Guid.NewGuid().ToString();
            
            CssStylesheetInfoProvider.SetCssStylesheetInfo(si);
        }
    }


    /// <summary>
    /// Undo the CSS stylesheet check out
    /// </summary>
    /// <param name="cssStylesheetId">Stylesheet ID</param>
    public static void UndoCheckOutStylesheet(int cssStylesheetId)
    {
        CssStylesheetInfo si = CssStylesheetInfoProvider.GetCssStylesheetInfo(cssStylesheetId);
        if (si != null)
        {
            string filename = HttpContext.Current.Server.MapPath(si.StylesheetCheckedOutFilename);
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }

            si.StylesheetCheckedOutByUserID = 0;
            si.StylesheetCheckedOutFilename = "";
            si.StylesheetCheckedOutMachineName = "";
            si.StylesheetVersionGUID = Guid.NewGuid().ToString();
            
            CssStylesheetInfoProvider.SetCssStylesheetInfo(si);
        }
    }

    #endregion


    #region "Layout management"

    /// <summary>
    /// Checks out the specified layout to the disk file
    /// </summary>
    /// <param name="layoutId">Layout ID</param>
    public static void CheckOutLayout(int layoutId)
    {
        LayoutInfo li = LayoutInfoProvider.GetLayoutInfo(layoutId);
        if (li != null)
        {
            string filename = LayoutInfoProvider.GetLayoutUrl(li.LayoutCodeName, null);

            SaveCodeFile(filename, LayoutInfoProvider.GetLayoutDirectives() + li.LayoutCode);
            
            // Set the layout data
            li.LayoutCheckedOutByUserID = CMSContext.CurrentUser.UserID;
            li.LayoutCheckedOutMachineName = HttpContext.Current.Server.MachineName;
            li.LayoutCheckedOutFilename = filename;
            li.LayoutVersionGUID = Guid.NewGuid().ToString();

            LayoutInfoProvider.SetLayoutInfo(li);
        }
    }


    /// <summary>
    /// Checks in the specified layout from the disk file
    /// </summary>
    /// <param name="layoutId">Layout ID</param>
    public static void CheckInLayout(int layoutId)
    {
        LayoutInfo li = LayoutInfoProvider.GetLayoutInfo(layoutId);
        if (li != null)
        {
            string filename = HttpContext.Current.Server.MapPath(li.LayoutCheckedOutFilename);
            TextReader tr = new StreamReader(filename);

            // Read away the directive lines
            int skiplines = LayoutInfoProvider.GetLayoutDirectives().Split('\n').Length - 1;
            for (int i = 0; i < skiplines; i++)
            {
                tr.ReadLine();
            }

            string newcode = tr.ReadToEnd();
            tr.Close();
            File.Delete(filename);

            li.LayoutCheckedOutByUserID = 0;
            li.LayoutCheckedOutFilename = "";
            li.LayoutCheckedOutMachineName = "";
            li.LayoutCode = newcode;
            li.LayoutVersionGUID = Guid.NewGuid().ToString();
            
            LayoutInfoProvider.SetLayoutInfo(li);
        }
    }


    /// <summary>
    /// Undo the layout check out
    /// </summary>
    /// <param name="layoutId">Layout ID</param>
    public static void UndoCheckOutLayout(int layoutId)
    {
        LayoutInfo li = LayoutInfoProvider.GetLayoutInfo(layoutId);
        if (li != null)
        {
            string filename = HttpContext.Current.Server.MapPath(li.LayoutCheckedOutFilename);
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }

            li.LayoutCheckedOutByUserID = 0;
            li.LayoutCheckedOutFilename = "";
            li.LayoutCheckedOutMachineName = "";
            li.LayoutVersionGUID = Guid.NewGuid().ToString();
            
            LayoutInfoProvider.SetLayoutInfo(li);
        }
    }

    #endregion


    #region "Template layout management"

    /// <summary>
    /// Checks out the page template layout to the disk file
    /// </summary>
    /// <param name="templateId">Template ID</param>
    public static void CheckOutTemplateLayout(int templateId)
    {
        PageTemplateInfo pti = PageTemplateInfoProvider.GetPageTemplateInfo(templateId);
        if (pti != null)
        {
            string filename = PageTemplateInfoProvider.GetLayoutUrl(pti.CodeName, null);

            SaveCodeFile(filename, LayoutInfoProvider.GetLayoutDirectives() + pti.PageTemplateLayout);

            // Set the layout data
            pti.PageTemplateLayoutCheckedOutByUserID = CMSContext.CurrentUser.UserID;
            pti.PageTemplateLayoutCheckedOutMachineName = HttpContext.Current.Server.MachineName;
            pti.PageTemplateLayoutCheckedOutFileName = filename;
            pti.PageTemplateVersionGUID = Guid.NewGuid().ToString();

            PageTemplateInfoProvider.SetPageTemplateInfo(pti);
        }
    }


    /// <summary>
    /// Checks in the page template layout from the disk file
    /// </summary>
    /// <param name="templateId">Template ID</param>
    public static void CheckInTemplateLayout(int templateId)
    {
        PageTemplateInfo pti = PageTemplateInfoProvider.GetPageTemplateInfo(templateId);
        if (pti != null)
        {
            string filename = HttpContext.Current.Server.MapPath(pti.PageTemplateLayoutCheckedOutFileName);
            TextReader tr = new StreamReader(filename);

            // Read away the directive lines
            int skiplines = LayoutInfoProvider.GetLayoutDirectives().Split('\n').Length - 1;
            for (int i = 0; i < skiplines; i++)
            {
                tr.ReadLine();
            }

            string newcode = tr.ReadToEnd();
            tr.Close();
            File.Delete(filename);

            pti.PageTemplateLayoutCheckedOutByUserID = 0;
            pti.PageTemplateLayoutCheckedOutFileName = "";
            pti.PageTemplateLayoutCheckedOutMachineName = "";
            pti.PageTemplateLayout = newcode;
            pti.PageTemplateVersionGUID = Guid.NewGuid().ToString();

            PageTemplateInfoProvider.SetPageTemplateInfo(pti);
        }
    }


    /// <summary>
    /// Undo check out of the page template layout
    /// </summary>
    /// <param name="templateId">Template ID</param>
    public static void UndoCheckOutTemplateLayout(int templateId)
    {
        PageTemplateInfo pti = PageTemplateInfoProvider.GetPageTemplateInfo(templateId);
        if (pti != null)
        {
            string filename = HttpContext.Current.Server.MapPath(pti.PageTemplateLayoutCheckedOutFileName);
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }

            pti.PageTemplateLayoutCheckedOutByUserID = 0;
            pti.PageTemplateLayoutCheckedOutFileName = "";
            pti.PageTemplateLayoutCheckedOutMachineName = "";
            pti.PageTemplateVersionGUID = Guid.NewGuid().ToString();

            PageTemplateInfoProvider.SetPageTemplateInfo(pti);
        }
    }

    #endregion



    /// <summary>
    /// Saves the code file to the disk
    /// </summary>
    /// <param name="filename">File name</param>
    /// <param name="content">File content</param>
    public static void SaveCodeFile(string filename, string content)
    {
        string fullfilename = HttpContext.Current.Server.MapPath(filename);
        DirectoryHelper.EnsureDiskPath(fullfilename, HttpContext.Current.Server.MapPath("~/"));

        TextWriter sw = new StreamWriter(fullfilename, false);
        content = HTMLHelper.EnsureLineEnding(content, "\r\n");
        sw.Write(content);
        sw.Close();
    }
}
