using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Drawing.Imaging;
using System.Drawing;
using System.IO;

using CMS.FileManager;
using CMS.WorkflowEngine;
using CMS.TreeEngine;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.DataEngine;
using CMS.SiteProvider;
using CMS.SettingsProvider;
using CMS.PortalEngine;
using CMS.URLRewritingEngine;

public partial class CMSPages_GetFile : System.Web.UI.Page
{
    /// <summary>
    /// Set to false to disable the client caching
    /// </summary>
    protected bool useClientCache = true;

    protected int versionHistoryID = 0;
    protected int width = 0;
    protected int height = 0;
    protected int maxSideSize = 0;

    protected CMSOutputFile outputFile = null;

    protected CurrentUserInfo currentUser = null;
    protected CurrentSiteInfo currentSite = null;

    protected void Page_Load(object sender, EventArgs e)
    {
        // Get the site
        currentSite = CMSContext.CurrentSite;
        if (currentSite == null)
        {
            throw new Exception("Site not running.");
        }
        currentUser = CMSContext.CurrentUser;

        string useCacheItemName = CMSContext.BaseCacheKey + "|" + this.Context.Request.Url.ToString();

        // Seek the attachment in cache
        if ((CacheHelper.Contains(useCacheItemName)) && (CacheHelper.CacheImageEnabled(currentSite.SiteName)) && (currentUser.ViewMode == ViewModeEnum.LiveSite))
        {
            // Get OutputFile object and process it.
            outputFile = (CMSOutputFile)CacheHelper.GetItem(useCacheItemName);
            SendFile(outputFile);
        }
        else
        {
            // Get attachment, process it and store it in cache if caching enabled.
            ProcessAttachment();

            // Save to cache if enabled and live site mode
            if (CacheHelper.CacheImageEnabled(currentSite.SiteName) && (currentUser.ViewMode == ViewModeEnum.LiveSite))
            {
                CacheHelper.Add(useCacheItemName, outputFile, null, DateTime.Now.AddMinutes(CacheHelper.CacheImageMinutes(currentSite.SiteName)), System.Web.Caching.Cache.NoSlidingExpiration);
            }

            // Send the data
            SendFile(outputFile);
        }
    }


    /// <summary>
    /// Sends the given file within response
    /// </summary>
    /// <param name="file">File to send</param>
    protected void SendFile(CMSOutputFile file)
    {
        // Clear response.
        Response.Cookies.Clear();
        Response.Clear();

        Response.Cache.SetRevalidation(HttpCacheRevalidation.AllCaches);

        // Send the file
        if ((file != null) && (file.OutputData != null))
        {
            // Check authentication if secured file
            if (file.IsSecured)
            {
                URLRewriter.CheckSecured();
            }

            // Client caching - only on the live site
            if ( useClientCache && (currentUser.ViewMode == ViewModeEnum.LiveSite) && (CacheHelper.CacheImageEnabled(currentSite.SiteName)) )
            {
                // Determine the last modified date and etag sent from the browser
                string ifModifiedString = Request.Headers["If-Modified-Since"];
                if (ifModifiedString != null)
                {
                    // IE-browsers send something like this:
                    // If-Modified-Since: Wed, 19 Jul 2006 11:19:59 GMT; length=3350
                    DateTime ifModified;
                    if (DateTime.TryParse(ifModifiedString.Split(";".ToCharArray())[0], out ifModified))
                    {
                        // If not changed, let the browser use the cached data
                        if (file.LastModified <= ifModified.AddSeconds(1))
                        {
                            Response.StatusCode = (int)System.Net.HttpStatusCode.NotModified;
                            Response.End();
                            return;
                        }
                    }
                }
            }

            // Prepare response
            Response.ContentType = file.Attachment.AttachmentMimeType;
            SetDisposition(file.Attachment.AttachmentName, file.Attachment.AttachmentExtension);

            if ( useClientCache && (currentUser.ViewMode == ViewModeEnum.LiveSite) && (CacheHelper.CacheImageEnabled(currentSite.SiteName)))
            {
                // Send last modified header to allow client caching
                Response.Cache.SetLastModified(file.LastModified);
                Response.Cache.SetExpires(DateTime.Now.AddHours(24));
                if (!file.IsSecured)
                {
                    Response.Cache.SetCacheability(HttpCacheability.Public);
                }
            }
            
            // Add the file data
            Response.OutputStream.Write(file.OutputData, 0, file.OutputData.Length);
        }

        Response.End();
    }


    /// <summary>
    /// Processes the attachment.
    /// </summary>
    protected void ProcessAttachment()
    {
        Guid guid = ValidationHelper.GetGuid(Request.QueryString["guid"], Guid.Empty);
        Guid nodeguid = ValidationHelper.GetGuid(Request.QueryString["nodeguid"], Guid.Empty);

        versionHistoryID = ValidationHelper.GetInteger(Request.QueryString["versionhistoryid"], 0);
        width = ValidationHelper.GetInteger(Request.QueryString["width"], 0);
        height = ValidationHelper.GetInteger(Request.QueryString["height"], 0);
        maxSideSize = ValidationHelper.GetInteger(Request.QueryString["maxSideSize"], 0);

        // If guid given, process the attachment
        if (guid != Guid.Empty)
        {
            if (versionHistoryID > 0)
            {
                ProcessFile(guid, versionHistoryID);
            }
            else
            {
                ProcessFile(guid);
            }
        }
        else if (nodeguid != Guid.Empty)
        {
            // If node GUID given, process the file
            ProcessNode(nodeguid);
        }
    }


    /// <summary>
    /// Set rate if width is empty and height not or reversely
    /// </summary>
    private void SetRate(int pictureWidth, int pictureHeight)
    {
        if ((pictureWidth == 0) && (pictureHeight == 0))
            return;

        if (width > pictureWidth)
            width = pictureWidth;

        if (height > pictureHeight)
            height = pictureHeight;

        float ratio = 0;

        if ((width > 0) && (height == 0))
        {
            ratio = (float)pictureWidth / (float)width;
            height = (int)((float)pictureHeight / (float)ratio);
        }

        if ((width == 0) && (height > 0))
        {
            ratio = (float)pictureHeight / (float)height;
            width = (int)((float)pictureWidth / (float)ratio);
        }
    }


    /// <summary>
    /// Processes the specified file and returns the data to the output stream
    /// </summary>
    /// <param name="attachmentGuid">Attachment guid.</param>
    protected void ProcessFile(Guid attachmentGuid)
    {
        AttachmentInfo atInfo = null;
        GeneralConnection conn = ConnectionHelper.GetConnection();
        AttachmentManager atManager = new AttachmentManager(conn);

        // Get AttachmentInfo object
        if (currentUser.ViewMode != ViewModeEnum.LiveSite)
        {
            // Not livesite mode - get latest version
            TreeProvider tree = new TreeProvider(currentUser, conn);
            atInfo = DocumentHelper.GetAttachment(attachmentGuid, tree, currentSite.SiteName);
        }
        else
        {
            // Live site mode 
            //atInfo = atManager.GetAttachmentInfoWithoutBinary(attachmentGuid, CMSContext.CurrentSite.SiteName);
            atInfo = atManager.GetAttachmentInfo(attachmentGuid, currentSite.SiteName);
        }

        if (atInfo != null)
        {
            // Check secured areas
            TreeProvider tp = new TreeProvider();
            TreeNode tn = DocumentHelper.GetDocument(atInfo.AttachmentDocumentID, tp);
            bool secured = false;
            if (tn != null)
            {
                // Get the site
                SiteInfo si = SiteInfoProvider.GetSiteInfo(tn.NodeSiteID);
                if (si != null)
                {
                    // Get the page info
                    PageInfo pi = PageInfoProvider.GetPageInfo(si.SiteName, tn.NodeAliasPath, tn.DocumentCulture, tn.DocumentUrlPath, false);
                    if (pi != null)
                    {
                        CMS.URLRewritingEngine.URLRewriter.RequestSecurePage(pi, false);

                        secured = pi.RequiresAuthentication;
                        CMS.URLRewritingEngine.URLRewriter.CheckSecuredAreas(pi, false);
                    }
                }
            }

            // Get the data
            byte[] mFile = null;
            if (atInfo.AttachmentBinary != null)
            {
                mFile = atInfo.AttachmentBinary;
            }
            else
            {
                mFile = atManager.GetFile(attachmentGuid, currentSite.SiteName);
            }

            if (mFile != null)
            {
                // Resize the image
                if (ImageHelper.IsImage(atInfo.AttachmentExtension))
                {
                    mFile = atManager.GetImageThumbnail(attachmentGuid, mFile, atInfo.AttachmentExtension, currentSite.SiteName, width, height, maxSideSize);
                }

                // Create the output file
                if (mFile != null)
                {
                    outputFile = new CMSOutputFile(atInfo, mFile);
                    outputFile.IsSecured = secured;
                }
            }
        }
    }


    /// <summary>
    /// Processes the specified document node
    /// </summary>
    /// <param name="nodeGuid">Node GUID</param>
    protected void ProcessNode(Guid nodeGuid)
    {
        string siteName = currentSite.SiteName;
        TreeProvider tree = new TreeProvider(currentUser);

        TreeNode node = tree.SelectSingleNode(nodeGuid, TreeProvider.ALL_CULTURES, siteName);
        if (node != null)
        {
            node = tree.SelectSingleNode(siteName, node.NodeAliasPath, currentUser.PreferredCultureCode, SettingsKeyProvider.GetBoolValue(siteName + ".CMSCombineImagesWithDefaultCulture"), "CMS.File", false);

            // Get the document data
            if (currentUser.ViewMode != ViewModeEnum.LiveSite)
            {
                node = DocumentHelper.GetDocument(node, tree);
            }

            if (node != null)
            {
                // Check secured area
                SiteInfo si = SiteInfoProvider.GetSiteInfo(node.NodeSiteID);
                if (si != null)
                {
                    PageInfo pi = PageInfoProvider.GetPageInfo(si.SiteName, node.NodeAliasPath, node.DocumentCulture, node.DocumentUrlPath, false);
                    if (pi != null)
                    {
                        CMS.URLRewritingEngine.URLRewriter.RequestSecurePage(pi, false);
                        CMS.URLRewritingEngine.URLRewriter.CheckSecuredAreas(pi, false);
                    }
                }

                if (node.NodeClassName.ToLower() == "cms.file")
                {
                    Guid attachmentGuid = ValidationHelper.GetGuid(node.GetValue("FileAttachment"), Guid.Empty);
                    if (attachmentGuid != Guid.Empty)
                    {
                        ProcessFile(attachmentGuid);
                    }
                }
            }
        }
    }


    /// <summary>
    /// Processes the specified version of the file and returns the data to the output stream
    /// </summary>
    /// <param name="attachmentGuid">Attachment GUID</param>
    /// <param name="versionHistoryId">Document version history ID</param>
    protected void ProcessFile(Guid attachmentGuid, int versionHistoryId)
    {
        AttachmentInfo atInfo = GetFile(attachmentGuid, versionHistoryId);

        if (atInfo != null)
        {
            TreeProvider tp = new TreeProvider();
            TreeNode tn = tp.SelectSingleDocument(atInfo.AttachmentDocumentID);

            if (tn != null)
            {
                // Check secured area
                SiteInfo si = SiteInfoProvider.GetSiteInfo(tn.NodeSiteID);
                if (si != null)
                {
                    PageInfo pi = PageInfoProvider.GetPageInfo(si.SiteName, tn.NodeAliasPath, tn.DocumentCulture, tn.DocumentUrlPath, false);
                    if (pi != null)
                    {
                        CMS.URLRewritingEngine.URLRewriter.RequestSecurePage(pi, false);
                        CMS.URLRewritingEngine.URLRewriter.CheckSecuredAreas(pi, false);
                    }
                }

                if ((currentUser.IsAuthorizedPerTreeNode(tn.NodeID, NodePermissionsEnum.Read) != AuthorizationResultEnum.Denied) ||
                     (currentUser.IsAuthorizedPerResource("cms.content", "read")))
                {
                    // If attachment is image, try resize
                    byte[] mFile = atInfo.AttachmentBinary;

                    if (mFile != null)
                    {
                        if (ImageHelper.IsImage(atInfo.AttachmentExtension))
                        {
                            AttachmentManager atManager = new AttachmentManager(tp.Connection);
                            mFile = atManager.GetImageThumbnail(attachmentGuid, mFile, atInfo.AttachmentExtension, currentSite.SiteName, width, height, maxSideSize);
                        }

                        if (mFile != null)
                        {
                            outputFile = new CMSOutputFile(atInfo, mFile);
                        }
                    }
                }
            }
        }
    }


    /// <summary>
    /// Gets the file from version history
    /// </summary>
    /// <param name="attachmentGuid">Atachment GUID</param>
    /// <param name="versionHistoryId">Version history ID</param>
    protected AttachmentInfo GetFile(Guid attachmentGuid, int versionHistoryId)
    {
        TreeProvider tree = new TreeProvider(currentUser);
        VersionManager vm = new VersionManager(tree);
        DataClass dc = vm.GetAttachmentVersion(versionHistoryId, attachmentGuid);
        if ((dc == null) || (dc.IsEmpty()))
        {
            return null;
        }
        else
        {
            return new AttachmentInfo(dc.DataRow, tree.Connection);
        }
    }


    /// <summary>
    /// Sets response header according to file type
    /// </summary>
    /// <param name="extension">File extension.</param>
    protected void SetDisposition(string fileName, string extension)
    {
        string disposition = ValidationHelper.GetString(Request.QueryString["disposition"], "");


        // image or html/htm
        if (ImageHelper.IsImage(extension) || ImageHelper.IsHtml(extension) || extension.ToLower() == ".pdf")
        {
            // Disposition type - For images "inline" is default
            if (disposition.ToLower() == "attachment")
            {
                Response.AddHeader("Content-Disposition", "attachment; filename=" + FilenameEncode(fileName));
            }
            else
            {
                Response.AddHeader("Content-Disposition", "inline; filename=" + FilenameEncode(fileName));
            }

        }
        // other file type
        else
        {
            // Disposition type - For files "attachment" is default type
            if (disposition.ToLower() == "inline")
            {
                Response.AddHeader("Content-Disposition", "inline; filename=" + FilenameEncode(fileName));
            }
            else
            {
                Response.AddHeader("Content-Disposition", "attachment; filename=" + FilenameEncode(fileName));
            }
        }


    }


    /// <summary>File name enconding function</summary>
    /// <param name="filename">Filename to encode.</param>
    private string FilenameEncode(string filename)
    {
        return HttpUtility.UrlEncode(filename, Response.HeaderEncoding);
    }

}
